--
--  Copyright (C) 2006-2007 Serotonin Software Technologies Inc.
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of version 2 of the GNU General Public License as 
--  published by the Free Software Foundation.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software Foundation, Inc.,
--  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
--
--  @author Matthew Lohbihler
--
--

--
-- System settings
create table systemSettings (
  settingName varchar(32) not null,
  settingValue varchar(255)
);
alter table systemSettings add constraint systemSettingsPk primary key (settingName);


--
-- Users
create table users (
  id int not null generated by default as identity (start with 1, increment by 1),
  username varchar(40) not null,
  password varchar(30) not null,
  email varchar(255) not null,
  phone varchar(40),
  admin char(1) not null,
  disabled char(1) not null,
  lastLogin bigint,
  selectedWatchList int
);
alter table users add constraint usersPk primary key (id);

create table userComments (
  userId int not null,
  commentType int not null,
  typeKey int not null,
  ts bigint not null,
  commentText varchar(1024) not null
);
alter table userComments add constraint userCommentsFk1 foreign key (userId) references users(id);


--
-- Mailing lists
create table mailingLists (
  id int not null generated by default as identity (start with 1, increment by 1),
  name varchar(40) not null
);
alter table mailingLists add constraint mailingListsPk primary key (id);

create table mailingListInactive (
  mailingListId int not null,
  inactiveInterval int not null
);
alter table mailingListInactive add constraint mailingListInactiveFk1 foreign key (mailingListId) 
  references mailingLists(id) on delete cascade;

create table mailingListMembers (
  mailingListId int not null,
  typeId int not null,
  userId int,
  address varchar(255)
);
alter table mailingListMembers add constraint mailingListMembersFk1 foreign key (mailingListId) 
  references mailingLists(id) on delete cascade;




--
--
-- Data Sources
--
create table dataSources (
  id int not null generated by default as identity (start with 1, increment by 1),
  name varchar(40) not null,
  dataSourceType int not null,
  data blob not null
);
alter table dataSources add constraint dataSourcesPk primary key (id);


-- Data source permissions
create table dataSourceUsers (
  dataSourceId int not null,
  userId int not null
);
alter table dataSourceUsers add constraint dataSourceUsersFk1 foreign key (dataSourceId) references dataSources(id);
alter table dataSourceUsers add constraint dataSourceUsersFk2 foreign key (userId) references users(id);



--
--
-- Data Points
--
create table dataPoints (
  id int not null generated by default as identity (start with 1, increment by 1),
  dataSourceId int not null,
  data blob not null
);
alter table dataPoints add constraint dataPointsPk primary key (id);
alter table dataPoints add constraint dataPointsFk1 foreign key (dataSourceId) references dataSources(id);


-- Data point permissions
create table dataPointUsers (
  dataPointId int not null,
  userId int not null,
  permission int not null
);
alter table dataPointUsers add constraint dataPointUsersFk1 foreign key (dataPointId) references dataPoints(id);
alter table dataPointUsers add constraint dataPointUsersFk2 foreign key (userId) references users(id);



--
--
-- Views
--
create table mangoViews (
  id int not null generated by default as identity (start with 1, increment by 1),
  name varchar(100) not null,
  background varchar(255),
  userId int not null,
  anonymousAccess int not null
);
alter table mangoViews add constraint mangoViewsPk primary key (id);
alter table mangoViews add constraint mangoViewsFk1 foreign key (userId) references users(id);

-- Point Views
create table pointViews (
  id int not null generated by default as identity (start with 1, increment by 1),
  mangoViewId int not null,
  dataPointId int not null,
  nameOverride varchar(255),
  settableOverride char(1) not null,
  bkgdColorOverride varchar(20),
  x int not null,
  y int not null,
  
  -- Graphic renderer
  grData blob not null
);
alter table pointViews add constraint pointViewsPk primary key (id);
alter table pointViews add constraint pointViewsFk1 foreign key (mangoViewId) references mangoViews(id);
alter table pointViews add constraint pointViewsFk2 foreign key (dataPointId) references dataPoints(id);



--
--
-- Point Values (historical data)
--
create table pointValues (
  id int not null generated by default as identity (start with 1, increment by 1),
  dataPointId int not null,
  dataType int not null,
  pointValue double,
  ts bigint not null
);
alter table pointValues add constraint pointValuesPk primary key (id);
alter table pointValues add constraint pointValuesFk1 foreign key (dataPointId) references dataPoints(id);
create index pointValuesIdx1 on pointValues (ts, dataPointId);
create index pointValuesIdx2 on pointValues (dataPointId, ts);

create table pointValueAnnotations (
  pointValueId int not null,
  textPointValueShort varchar(128),
  textPointValueLong clob,
  sourceType smallint,
  sourceId int
);
alter table pointValueAnnotations add constraint pointValueAnnotationsFk1 foreign key (pointValueId) 
  references pointValues(id) on delete cascade;


--
--
-- Watch list
--
create table watchLists (
  id int not null generated by default as identity (start with 1, increment by 1),
  userId int not null,
  name varchar(50)
);
alter table watchLists add constraint watchListsPk primary key (id);
alter table watchLists add constraint watchListsFk1 foreign key (userId) references users(id);

create table watchListPoints (
  watchListId int not null,
  dataPointId int not null,
  sortOrder int not null
);
alter table watchListPoints add constraint watchListPointsFk1 foreign key (watchListId) references watchLists(id);
alter table watchListPoints add constraint watchListPointsFk2 foreign key (dataPointId) references dataPoints(id);


--
--
-- Point event detectors
--
create table pointEventDetectors (
  id int not null generated by default as identity (start with 1, increment by 1),
  dataPointId int not null,
  detectorType int not null,
  alarmLevel int not null,
  limit double,
  duration int,
  durationType int,
  binaryState char(1),
  multistateState int,
  changeCount int,
  alphanumericState varchar(128)
);
alter table pointEventDetectors add constraint pointEventDetectorsPk primary key (id);
alter table pointEventDetectors add constraint pointEventDetectorsFk1 foreign key (dataPointId) 
  references dataPoints(id);


--
--
-- Events
--
create table events (
  id int not null generated by default as identity (start with 1, increment by 1),
  typeId int not null,
  typeRef1 int not null,
  typeRef2 int not null,
  activeTs bigint not null,
  inactiveTs bigint,
  inactiveCause int,
  inactiveCauseRef int,
  rtnApplicable char(1) not null,
  alarmLevel int not null,
  message varchar(1024)
);
alter table events add constraint eventsPk primary key (id);


--
--
-- Event handlers
--
create table eventHandlers (
  id int not null generated by default as identity (start with 1, increment by 1),
  
  -- Event type, see events
  eventTypeId int not null,
  eventTypeRef1 int not null,
  eventTypeRef2 int not null,
  
  data blob not null
);
alter table eventHandlers add constraint eventHandlersPk primary key (id);


--
--
-- Scheduled events
--
create table scheduledEvents (
  id int not null generated by default as identity (start with 1, increment by 1),
  alarmLevel int not null,
  scheduleType int not null,
  returnToNormal char(1) not null,
  disabled char(1) not null,
  activeYear int,
  activeMonth int,
  activeDay int,
  activeHour int,
  activeMinute int,
  activeSecond int,
  activeCron varchar(25),
  inactiveYear int,
  inactiveMonth int,
  inactiveDay int,
  inactiveHour int,
  inactiveMinute int,
  inactiveSecond int,
  inactiveCron varchar(25)
);
alter table scheduledEvents add constraint scheduledEventsPk primary key (id);


--
--
-- Point Hierarchy
--
create table pointHierarchy (
  id int not null generated by default as identity (start with 1, increment by 1),
  parentId int,
  name varchar(100)
);
alter table pointHierarchy add constraint pointHierarchyPk primary key (id);


--
--
-- Compound events detectors
--
create table compoundEventDetectors (
  id int not null generated by default as identity (start with 1, increment by 1),
  name varchar(100),
  alarmLevel int not null,
  returnToNormal char(1) not null,
  disabled char(1) not null,
  condition varchar(256) not null
);
alter table compoundEventDetectors add constraint compoundEventDetectorsPk primary key (id);


--
--
-- Reports
--
create table reports (
  id int not null generated by default as identity (start with 1, increment by 1),
  userId int not null,
  name varchar(100) not null,
  data blob not null
);
alter table reports add constraint reportsPk primary key (id);
alter table reports add constraint reportsFk1 foreign key (userId) references users(id);

create table reportInstances (
  id int not null generated by default as identity (start with 1, increment by 1),
  userId int not null,
  name varchar(100) not null,
  reportStartTime bigint not null,
  reportEndTime bigint not null,
  runStartTime bigint,
  runEndTime bigint,
  recordCount int,
  preventPurge char(1)
);
alter table reportInstances add constraint reportInstancesPk primary key (id);
alter table reportInstances add constraint reportInstancesFk1 foreign key (userId) references users(id);

create table reportInstancePoints (
  id int not null generated by default as identity (start with 1, increment by 1),
  reportInstanceId int not null,
  pointName varchar(100) not null,
  dataType int not null,
  startValue varchar(4096),
  textRenderer blob
);
alter table reportInstancePoints add constraint reportInstancePointsPk primary key (id);
alter table reportInstancePoints add constraint reportInstancePointsFk1 foreign key (reportInstanceId) 
  references reportInstances(id) on delete cascade;

create table reportInstanceData (
  pointValueId int not null,
  reportInstancePointId int not null,
  pointValue double,
  ts bigint not null
);
alter table reportInstanceData add constraint reportInstanceDataPk primary key (pointValueId, reportInstancePointId);
alter table reportInstanceData add constraint reportInstanceDataFk1 foreign key (reportInstancePointId) 
  references reportInstancePoints(id) on delete cascade;

create table reportInstanceDataAnnotations (
  pointValueId int not null,
  reportInstancePointId int not null,
  textPointValueShort varchar(128),
  textPointValueLong clob,
  sourceValue varchar(128)
);
alter table reportInstanceDataAnnotations add constraint reportInstanceDataAnnotationsPk 
  primary key (pointValueId, reportInstancePointId);
alter table reportInstanceDataAnnotations add constraint reportInstanceDataAnnotationsFk1 
  foreign key (pointValueId, reportInstancePointId) references reportInstanceData(pointValueId, reportInstancePointId) 
  on delete cascade;




--
--
-- Publishers
--
create table publishers (
  id int not null generated by default as identity (start with 1, increment by 1),
  data blob not null
);
alter table publishers add constraint publishersPk primary key (id);
