/*
  Copyright (C) 2006-2007 Serotonin Software Technologies Inc.

  This program is free software; you can redistribute it and/or modify
  it under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

  @author Matthew Lohbihler
*/

var mango = {};

//
// String prototypes
//
String.prototype.startsWith = function(str) {
    if (str.length > this.length)
        return false;
    for (var i=0; i<str.length; i++) {
        if (str.charAt(i) != this.charAt(i))
            return false;
    }
    return true;
}

String.prototype.trim = function() {
    return this.replace(/^\s+|\s+$/g,"");
}

//
// Custom exception to string
//
function errorToString(e) {
    try {
        return e.name +": "+ e.message +" ("+ e.fileName +":"+ e.lineNumber +")";
    }
    catch (e2) {
        return e.name +": "+ e.message +" ("+ e.fileName +")";
    }
}

//
// Input control
//
function setDisabled(node, disabled) {
    node = getNodeIfString(node);
    if (disabled) {
        node.disabled = true;
        dojo.html.addClass(node, "formDisabled");
    }
    else {
        node.disabled = false;
        dojo.html.removeClass(node, "formDisabled");
    }
}

function dump(o) {
    for (var p in o)
        dojo.debug(p +"="+ o[p]);
}

function contains(arr, e) {
    for (var i=0; i<arr.length; i++) {
        if (arr[i] == e)
            return true;
    }
    return false;
}

//
// Common functions (delegates to Dojo functions)
//
function show(node) {
    dojo.html.show(node);
}

function hide(node) {
    try {
        dojo.html.hide(node);
    } catch (err) {
        throw "hide failed for node "+ node +", "+ err.message;
    }
}

function display(node, showNode) {
    if (showNode)
        dojo.html.show(node);
    else
        dojo.html.hide(node);
}

function showMenu(node, xoffset, yoffset) {
    node = getNodeIfString(node);
    var bounds = dojo.html.abs(node.parentNode);
    var anc = findRelativeAncestor(node);
    if (anc) {
        var rbounds = dojo.html.abs(anc);
        bounds.x -= rbounds.x;
        bounds.y -= rbounds.y;
    }
    node.style.left = (bounds.x + xoffset) +"px";
    node.style.top = (bounds.y + yoffset) +"px";
    dojo.html.setVisibility(node, true);
}

function showLayer(node) {
    dojo.html.setVisibility(node, true);
}

function hideLayer(node) {
    dojo.html.setVisibility(node, false);
}

function findRelativeAncestor(node) {
    var pos;
    while (node = node.parentNode) {
        pos = dojo.html.getStyle(node, "position");
        if (pos == "relative" || pos == "absolute")
            return node;
    }
    return null;
}


//
// Get the dimensions of an element. Uses Dojo functions. Assumes a content box model, which may not be correct
// in all situations.
//
function getNodeBounds(node) {
    node = getNodeIfString(node);
    //dojo.debug(dojo.html.getBoxSizing(node));
    var box = dojo.html.getContentBox(node);
    return {
        x : dojo.html.getPixelValue(node, "left", dojo.html.isPositionAbsolute(node)),
        y : dojo.html.getPixelValue(node, "top", dojo.html.isPositionAbsolute(node)),
        w : box.width,
        h : box.height
    };
}

function getAbsoluteNodeBounds(node) {
    //dojo.debug(dojo.html.getBoxSizing(node));
    var box = dojo.html.getContentBox(node);
    var x = y = 0;
    var tempNode = node;
    while (tempNode) {
        x += tempNode.offsetLeft;
        y += tempNode.offsetTop;
        tempNode = tempNode.offsetParent;
    }
    return {
        x : x,
        y : y,
        w : box.width,
        h : box.height
    };
}


//
// Class for fading images in and out. Requires Dojo.
//
function IEBlinker(/*element*/node, /*milliseconds*/onTime, /*milliseconds*/offTime) {
    this.target = node;
    this.on = onTime;
    if (!this.on)
        this.on = 700;
    this.off = offTime;
    if (!this.off)
        this.off = 300;
    
    this.state = true;
    this.timeoutId;
    this.started = false;
    
    this.start = function() {
        this.started = true;
        this.timeoutId = null;
        this.state = !this.state;
        dojo.html.setVisibility(this.target, this.state);
        this.timeoutId = setTimeout(dojo.lang.hitch(this, "start"), this.state ? this.on : this.off);
    };
    
    this.stop = function() {
        if (this.started) {
            this.started = false;
            clearTimeout(this.timeoutId);
            this.timeoutId = null;
        }
        dojo.html.setVisibility(this.target, true);
    };
}

function startIEBlinker(node) {
    node = getNodeIfString(node);
    var blinker = new IEBlinker(node);
    if (node.blinker)
        stopIEBlinker(node);
    node.blinker = blinker;
    blinker.start();
}

function hasIEBlinker(node) {
    node = getNodeIfString(node);
    if (node.blinker)
        return true;
    return false;
}

function stopIEBlinker(node) {
    node = getNodeIfString(node);
    var blinker = node.blinker;
    if (blinker) {
        blinker.stop();
        node.blinker = null;
    }
}


function ImageFader(/*element*/imgNode, /*milliseconds*/cycleRate, /*0<float<1*/cycleStep) {
    this.im = imgNode;
    this.rate = cycleRate;
    if (!this.rate)
        this.rate = 100;
    this.step = cycleStep;
    if (!this.step)
        this.step = 0.1;
    
    this.increasing = false;
    this.timeoutId;
    this.started = false;
    
    this.start = function() {
        this.started = true;
        this.timeoutId = null;
        
        var op = dojo.html.getOpacity(this.im, "opacity");
        if (op >= 1)
            this.increasing = false;
        else if (op <= 0)
            this.increasing = true;
    
        if (this.increasing)
            op += this.step;
        else
            op -= this.step;
        
        dojo.html.setOpacity(this.im, op);
    
        this.timeoutId = setTimeout(dojo.lang.hitch(this, "start"), this.rate);
    };
    
    this.stop = function() {
        if (this.started) {
            this.started = false;
            clearTimeout(this.timeoutId);
            this.timeoutId = null;
        }
        dojo.html.setOpacity(this.im, 1);
    };
}

function startImageFader(node, disableOnclick) {
    if (disableOnclick)
        this.disableOnclick(node);
    
    node = getNodeIfString(node);
    var fader = new ImageFader(node);
    if (node.fader)
        stopImageFader(node);
    node.fader = fader;
    fader.start();
}

function hasImageFader(node) {
    node = getNodeIfString(node);
    if (node.fader)
        return true;
    return false;
}

function stopImageFader(node) {
    enableOnclick(node);
    node = getNodeIfString(node);
    var fader = node.fader;
    if (fader) {
        fader.stop();
        node.fader = null;
    }
}

function disableOnclick(node) {
    node.disabledOnclick = node.onclick;
    node.onclick = null;
}

function enableOnclick(node) {
    if (node.disabledOnclick) {
        node.onclick = node.disabledOnclick;
        node.disabledOnclick = null;;
    }
}

function updateTemplateNode(elem, replaceText) {
    var i;
    for (i=0; i<elem.attributes.length; i++) {
        if (elem.attributes[i].value && elem.attributes[i].value.indexOf('_TEMPLATE_') != -1)
            elem.attributes[i].value = elem.attributes[i].value.replace(/_TEMPLATE_/, replaceText);
    }
    for (var i=0; i<elem.childNodes.length; i++) {
        if (elem.childNodes[i].attributes)
            updateTemplateNode(elem.childNodes[i], replaceText);
    }
}

function getElementsByMangoName(node, mangoName, result) {
    if (!result)
        result = new Array();
    if (node.mangoName == mangoName)
        result[result.length] = node;
    for (var i=0; i<node.childNodes.length; i++)
        getElementsByMangoName(node.childNodes[i], mangoName, result);
    return result;
}

function createFromTemplate(templateId, id, parentId) {
    var content = $(templateId).cloneNode(true);
    updateTemplateNode(content, id);
    content.mangoId = id;
    $(parentId).appendChild(content);
    show(content);
    return content;
}

function getMangoId(node) {
    while (!(node.mangoId))
        node = node.parentNode;
    return node.mangoId;
}

function setAlarmLevelImg(alarmLevel, imgNode) {
    if (alarmLevel == 0)
        updateImg(imgNode, "images/flag_green.png", "None", false);
    else if (alarmLevel == 1)
        updateImg(imgNode, "images/flag_blue.png", "Information", true);
    else if (alarmLevel == 2)
        updateImg(imgNode, "images/flag_yellow.png", "Urgent", true);
    else if (alarmLevel == 3)
        updateImg(imgNode, "images/flag_orange.png", "Critical", true);
    else if (alarmLevel == 4)
        updateImg(imgNode, "images/flag_red.png", "Life safety", true);
    else
        updateImg(imgNode, "(unknown)", "(unknown)", true);
}

function setAlarmLevelText(alarmLevel, textNode) {
    if (alarmLevel == 0)
        textNode.innerHTML = "";
    else if (alarmLevel == 1)
        textNode.innerHTML = "Information";
    else if (alarmLevel == 2)
        textNode.innerHTML = "Urgent";
    else if (alarmLevel == 3)
        textNode.innerHTML = "Critical";
    else if (alarmLevel == 4)
        textNode.innerHTML = "Life safety";
    else
        textNode.innerHTML = "Unknown alarm level: "+ alarmLevel;
}

function setUserImg(admin, disabled, imgNode) {
    if (disabled)
        updateImg(imgNode, "images/user_disabled.png", "Disabled", true);
    else if (admin)
        updateImg(imgNode, "images/user_suit.png", "Administrator", true);
    else
        updateImg(imgNode, "images/user_green.png", "User", true);
}

function setScheduledEventImg(disabled, imgNode) {
    if (disabled)
        updateImg(imgNode, "images/clock_disabled.png", "Disabled scheduled event", true);
    else
        updateImg(imgNode, "images/clock.png", "Scheduled event", true);
}

function setCompoundEventImg(disabled, imgNode) {
    if (disabled)
        updateImg(imgNode, "images/multi_bell_disabled.png", "Disabled compound event detector", true);
    else
        updateImg(imgNode, "images/multi_bell.png", "Compound event detector", true);
}

function setDataSourceStatusImg(enabled, imgNode) {
    if (enabled)
        updateImg(imgNode, "images/database_go.png", "Enabled - click to disable", true);
    else
        updateImg(imgNode, "images/database_stop.png", "Disabled - click to enable", true);
}

function setDataPointStatusImg(enabled, imgNode) {
    if (enabled)
        updateImg(imgNode, "images/brick_go.png", "Enabled - click to disable", true);
    else
        updateImg(imgNode, "images/brick_stop.png", "Disabled - click to enable", true);
}

function setPublisherStatusImg(enabled, imgNode) {
    if (enabled)
        updateImg(imgNode, "images/transmit_go.png", "Enabled - click to disable", true);
    else
        updateImg(imgNode, "images/transmit_stop.png", "Disabled - click to enable", true);
}

function updateImg(imgNode, src, text, visible) {
    if (visible) {
        show(imgNode);
        if (src)
            imgNode.src = src;
        if (text) {
            imgNode.title = text;
            imgNode.alt = text;
        }
    }
    else
        hide(imgNode);
}

// For panels that default as displayed
function togglePanelVisibility(img, panelId, visTitle, invisTitle) {
    var visible = true;
    if (!img.minimized)
        visible = false;
    togglePanelVisibilityImpl(img, panelId, visTitle, invisTitle, visible);
}

// For panels that default as hidden
function togglePanelVisibility2(img, panelId, visTitle, invisTitle) {
    var visible = true;
    if (img.minimized == false)
        visible = false;
    togglePanelVisibilityImpl(img, panelId, visTitle, invisTitle, visible);
}

function togglePanelVisibilityImpl(img, panelId, visTitle, invisTitle, visible) {
    if (!visible) {
        img.src = "images/arrow_out.png";
        img.alt = invisTitle || "Maximize";
        img.title = invisTitle || "Maximize";
        dojo.html.hide(panelId);
        img.minimized = true;
    }
    else {
        img.src = "images/arrow_in.png";
        img.alt = visTitle || "Minimize";
        img.title = visTitle || "Minimize";
        dojo.html.show(panelId);
        img.minimized = false;
    }
}

function $get(comp) {
    return dwr.util.getValue(comp);
}

function $set(comp, value) {
    return dwr.util.setValue(comp, value);
}

function getSelectionRange(node) {
    node.focus();
    if (typeof node.selectionStart != "undefined")
        // FF
        return { start: node.selectionStart, end: node.selectionEnd };
    if (!document.selection)
        return { start: 0, end: 0 };
    
    // IE
    var range = document.selection.createRange();
    var rangeCopy = range.duplicate();
    rangeCopy.moveToElementText(tt);
    rangeCopy.setEndPoint('EndToEnd', range);
    var start = rangeCopy.text.length - range.text.length;
    return { start: start, end: start + range.text.length };
}

function setSelectionRange(node, start, end) {
    if (node.setSelectionRange) {
        node.setSelectionRange(start, end);
        node.focus();
    }
    else {
        var range = node.createTextRange();
        range.move('character', start);
        range.moveEnd('character', end - start);
        range.select();
    }
}

function insertIntoTextArea(node, text) {
    if (document.selection) {
        // IE
        node.focus();
        document.selection.createRange().text = text;
    }
    else {
        var oldScrollTop = node.scrollTop;
        var range = getSelectionRange(node);
        var value = node.value;
        value = value.substring(0, range.start) + text + value.substring(range.end);
        node.value = value;
        node.setSelectionRange(range.start + text.length, range.start + text.length);
        node.scrollTop = oldScrollTop;
    }
}

// Convenience method. Returns the first element in the given array that has an id property the same as the given id.
function getElement(arr, id, idName) {
    if (!idName)
        idName = "id";

    for (var i=0; i<arr.length; i++) {
        if (arr[i][idName] == id)
            return arr[i];
    }
    return null;
}

function updateElement(arr, id, key, value, dobreak) {
    for (var i=0; i<arr.length; i++) {
        if (arr[i].id == id) {
            arr[i][key] = value;
            if (dobreak)
                return;
        }
    }
}

function removeElement(arr, id) {
    for (var i=arr.length-1; i>=0; i--) {
        if (arr[i].id == id)
            arr.splice(i, 1);
    }
}

function showMessage(node, msg) {
    node = getNodeIfString(node);
    if (msg) {
        show(node);
        node.innerHTML = msg;
    }
    else
        hide(node);
}
  
function getNodeIfString(node) {
    if (typeof(node) == "string")
        return $(node);
    return node;
}

function escapeQuotes(str) {
    if (!str)
        return "";
    return str.replace(/\'/g,"\\'");
}

function escapeDQuotes(str) {
    if (!str)
        return "";
    return str.replace(/\"/g,"\\\"");
}

function encodeQuotes(str) {
    if (!str)
        return "";
    return str.replace(/\'/g,"%27").replace(/\"/g,"%22");
}

function encodeHtml(str) {
    if (!str)
        return "";
    return str.replace(/</g,"&lt;");
}

function appendNewElement(/*string*/type, /*node*/parent) {
    var node = document.createElement(type);
    parent.appendChild(node);
    return node;
}

function writeImage(id, src, png, title, onclick) {
    var result = '<img class="ptr"';
    if (id)
        result += ' id="'+ id +'"';
    if (src)
        result += ' src="'+ src +'"';
    if (png && !src)
        result += ' src="images/'+ png +'.png"';
    if (title)
        result += ' alt="'+ title +'" title="'+ title +'"';
    result += ' onclick="'+ onclick +'"/>';
    return result;
}

function writeImageSQuote(id, src, png, title, onclick) {
    var result = "<img class='ptr'";
    if (id)
        result += " id='"+ id +"'";
    if (src)
        result += " src='"+ src +"'";
    if (png && !src)
        result += " src='images/"+ png +".png'";
    if (title)
        result += " alt='"+ title +"' title='"+ title +"'";
    result += " onclick='"+ onclick +"'/>";
    return result;
}

function hideContextualMessages(parent) {
    parent = getNodeIfString(parent);
    var nodes = dojo.html.getElementsByClass("ctxmsg", parent);
    for (var i=0; i<nodes.length; i++)
        dojo.html.hide(nodes[i]);
}

function createContextualMessageNode(field, fieldId) {
    field = getNodeIfString(field);
    var node = document.createElement("div");
    node.id = fieldId +"Ctxmsg";
    node.className = "ctxmsg formError";
    dojo.html.hide(node);
    
    var next = field.nextSibling;
    if (next)
        next.parentNode.insertBefore(node, next);
    else
        field.parentNode.appendChild(node);
    return node;
}

function showDwrMessages(/*DwrResponse.messages*/messages, /*tbody*/genericMessageNode) {
    var m, field, node, next;
    var genericMessages = new Array();
    for (var i=0; i<messages.length; i++) {
        m = messages[i];
        if (m.contextKey) {
            node = $(m.contextKey +"Ctxmsg");
            if (!node) {
                field = $(m.contextKey);
                if (field)
                    node = createContextualMessageNode(field, m.contextKey);
                else
                    alert("No contextual field found for key "+ m.contextKey);
            }
            
            if (node) {
                node.innerHTML = m.contextualMessage;
                dojo.html.show(node);
            }
        }
        else
            genericMessages[genericMessages.length] = m.genericMessage;
    }
    
    if (genericMessages.length > 0) {
        if (!genericMessageNode)
            alert("generic messages node not defined");
        genericMessageNode = getNodeIfString(genericMessageNode);
        dwr.util.removeAllRows(genericMessageNode);
        dwr.util.addRows(genericMessageNode, genericMessages, [ function(data) { return data; } ],
            {
                cellCreator:function(options) {
                    var td = document.createElement("td");
                    td.className = "formError";
                    return td;
                }
            });
        dojo.html.show(genericMessageNode);
    }
}